@@ -1,9 +1,14 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2019, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
+/**
+ * @file crypt_path_st.h
+ * @brief Path structures for origin circuits.
+ **/
+
 #ifndef CRYPT_PATH_ST_H
 #define CRYPT_PATH_ST_H
 
@@ -24,6 +29,8 @@ struct onion_handshake_state_t {
   } u;
 };
 
+struct congestion_control_t;
+
 /** Macro to encapsulate private members of a struct.
  *
  *  Renames 'x' to 'x_crypt_path_private_field'.
@@ -35,7 +42,7 @@ struct onion_handshake_state_t {
 /* Helper macro to access private members of a struct. */
 #define pvt_crypto CRYPT_PATH_PRIV_FIELD(crypto)
 
-#endif
+#endif /* defined(CRYPT_PATH_PRIVATE) */
 
 /** Holds accounting information for a single step in the layered encryption
  * performed by a circuit.  Used only at the client edge of a circuit. */
@@ -75,6 +82,9 @@ struct crypt_path_t {
   int deliver_window; /**< How many cells are we willing to deliver originating
                        * at this step? */
 
+  /** Congestion control info */
+  struct congestion_control_t *ccontrol;
+
   /*********************** Private members ****************************/
 
   /** Private member: Cryptographic state used for encrypting and
@@ -82,4 +92,4 @@ struct crypt_path_t {
   relay_crypto_t CRYPT_PATH_PRIV_FIELD(crypto);
 };
 
-#endif
+#endif /* !defined(CRYPT_PATH_ST_H) */
