@@ -1,7 +1,7 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2020, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -64,7 +64,6 @@
 #include "feature/nodelist/routerlist.h"
 #include "feature/nodelist/routerset.h"
 #include "feature/nodelist/torcert.h"
-#include "feature/rend/rendservice.h"
 #include "lib/encoding/binascii.h"
 #include "lib/err/backtrace.h"
 #include "lib/geoip/geoip.h"
@@ -135,6 +134,10 @@ typedef struct nodelist_t {
   /* Set of addresses that belong to nodes we believe in. */
   address_set_t *node_addrs;
 
+  /* Set of addresses + port that belong to nodes we know and that we don't
+   * allow network re-entry towards them. */
+  digestmap_t *reentry_set;
+
   /* The valid-after time of the last live consensus that initialized the
    * nodelist.  We use this to detect outdated nodelists that need to be
    * rebuilt using a newer consensus. */
@@ -362,7 +365,7 @@ node_set_hsdir_index(node_t *node, const networkstatus_t *ns)
   tor_assert(node);
   tor_assert(ns);
 
-  if (!networkstatus_is_live(ns, now)) {
+  if (!networkstatus_consensus_reasonably_live(ns, now)) {
     static struct ratelim_t live_consensus_ratelim = RATELIM_INIT(30 * 60);
     log_fn_ratelim(&live_consensus_ratelim, LOG_INFO, LD_GENERAL,
                    "Not setting hsdir index with a non-live consensus.");
@@ -447,40 +450,96 @@ node_addrs_changed(node_t *node)
 static void
 node_add_to_address_set(const node_t *node)
 {
-  if (!the_nodelist || !the_nodelist->node_addrs)
+  if (!the_nodelist ||
+      !the_nodelist->node_addrs || !the_nodelist->reentry_set)
     return;
 
-  /* These various address sources can be redundant, but it's likely faster
-   * to add them all than to compare them all for equality. */
+  /* These various address sources can be redundant, but it's likely faster to
+   * add them all than to compare them all for equality.
+   *
+   * For relays, we only add the ORPort in the addr+port set since we want to
+   * allow re-entry into the network to the DirPort so the self reachability
+   * test succeeds and thus the 0 value for the DirPort. */
 
   if (node->rs) {
     if (!tor_addr_is_null(&node->rs->ipv4_addr))
-      nodelist_add_addr_to_address_set(&node->rs->ipv4_addr);
+      nodelist_add_addr_to_address_set(&node->rs->ipv4_addr,
+                                       node->rs->ipv4_orport, 0);
     if (!tor_addr_is_null(&node->rs->ipv6_addr))
-      nodelist_add_addr_to_address_set(&node->rs->ipv6_addr);
+      nodelist_add_addr_to_address_set(&node->rs->ipv6_addr,
+                                       node->rs->ipv6_orport, 0);
   }
   if (node->ri) {
     if (!tor_addr_is_null(&node->ri->ipv4_addr))
-      nodelist_add_addr_to_address_set(&node->ri->ipv4_addr);
+      nodelist_add_addr_to_address_set(&node->ri->ipv4_addr,
+                                       node->ri->ipv4_orport, 0);
     if (!tor_addr_is_null(&node->ri->ipv6_addr))
-      nodelist_add_addr_to_address_set(&node->ri->ipv6_addr);
+      nodelist_add_addr_to_address_set(&node->ri->ipv6_addr,
+                                       node->ri->ipv6_orport, 0);
   }
   if (node->md) {
     if (!tor_addr_is_null(&node->md->ipv6_addr))
-      nodelist_add_addr_to_address_set(&node->md->ipv6_addr);
+      nodelist_add_addr_to_address_set(&node->md->ipv6_addr,
+                                       node->md->ipv6_orport, 0);
   }
 }
 
+/** Build a construction for the reentry set consisting of an address and port
+ * pair.
+ *
+ * If the given address is _not_ AF_INET or AF_INET6, then the item is an
+ * array of 0s.
+ *
+ * Return a pointer to a static buffer containing the item. Next call to this
+ * function invalidates its previous content. */
+static char *
+build_addr_port_item(const tor_addr_t *addr, const uint16_t port)
+{
+  /* At most 16 bytes are put in this (IPv6) and then 2 bytes for the port
+   * which is within the maximum of 20 bytes (DIGEST_LEN). */
+  static char data[DIGEST_LEN];
+
+  memset(data, 0, sizeof(data));
+  switch (tor_addr_family(addr)) {
+  case AF_INET:
+    memcpy(data, &addr->addr.in_addr.s_addr, 4);
+    break;
+  case AF_INET6:
+    memcpy(data, &addr->addr.in6_addr.s6_addr, 16);
+    break;
+  case AF_UNSPEC:
+    /* Leave the 0. */
+    break;
+  default:
+    /* LCOV_EXCL_START */
+    tor_fragile_assert();
+    /* LCOV_EXCL_STOP */
+  }
+
+  memcpy(data + 16, &port, sizeof(port));
+  return data;
+}
+
 /** Add the given address into the nodelist address set. */
 void
-nodelist_add_addr_to_address_set(const tor_addr_t *addr)
+nodelist_add_addr_to_address_set(const tor_addr_t *addr,
+                                 uint16_t or_port, uint16_t dir_port)
 {
   if (BUG(!addr) || tor_addr_is_null(addr) ||
       (!tor_addr_is_v4(addr) && !tor_addr_is_v6(addr)) ||
-      !the_nodelist || !the_nodelist->node_addrs) {
+      !the_nodelist || !the_nodelist->node_addrs ||
+      !the_nodelist->reentry_set) {
     return;
   }
   address_set_add(the_nodelist->node_addrs, addr);
+  if (or_port != 0) {
+    digestmap_set(the_nodelist->reentry_set,
+                  build_addr_port_item(addr, or_port), (void*) 1);
+  }
+  if (dir_port != 0) {
+    digestmap_set(the_nodelist->reentry_set,
+                  build_addr_port_item(addr, dir_port), (void*) 1);
+  }
 }
 
 /** Return true if <b>addr</b> is the address of some node in the nodelist.
@@ -497,6 +556,21 @@ nodelist_probably_contains_address(const tor_addr_t *addr)
   return address_set_probably_contains(the_nodelist->node_addrs, addr);
 }
 
+/** Return true if <b>addr</b> is the address of some node in the nodelist and
+ * corresponds also to the given port. If not, probably return false. */
+bool
+nodelist_reentry_contains(const tor_addr_t *addr, uint16_t port)
+{
+  if (BUG(!addr) || BUG(!port))
+    return false;
+
+  if (!the_nodelist || !the_nodelist->reentry_set)
+    return false;
+
+  return digestmap_get(the_nodelist->reentry_set,
+                       build_addr_port_item(addr, port)) != NULL;
+}
+
 /** Add <b>ri</b> to an appropriate node in the nodelist.  If we replace an
  * old routerinfo, and <b>ri_old_out</b> is not NULL, set *<b>ri_old_out</b>
  * to the previous routerinfo.
@@ -628,10 +702,14 @@ nodelist_set_consensus(const networkstatus_t *ns)
    * v6). Then we add the number of configured trusted authorities we have. */
   int estimated_addresses = smartlist_len(ns->routerstatus_list) *
                             get_estimated_address_per_node();
-  estimated_addresses += (get_n_authorities(V3_DIRINFO & BRIDGE_DIRINFO) *
+  estimated_addresses += (get_n_authorities(V3_DIRINFO | BRIDGE_DIRINFO) *
                           get_estimated_address_per_node());
+  /* Clear our sets because we will repopulate them with what this new
+   * consensus contains. */
   address_set_free(the_nodelist->node_addrs);
   the_nodelist->node_addrs = address_set_new(estimated_addresses);
+  digestmap_free(the_nodelist->reentry_set, NULL);
+  the_nodelist->reentry_set = digestmap_new();
 
   SMARTLIST_FOREACH_BEGIN(ns->routerstatus_list, routerstatus_t *, rs) {
     node_t *node = node_get_or_create(rs->identity_digest);
@@ -858,6 +936,8 @@ nodelist_free_all(void)
 
   address_set_free(the_nodelist->node_addrs);
   the_nodelist->node_addrs = NULL;
+  digestmap_free(the_nodelist->reentry_set, NULL);
+  the_nodelist->reentry_set = NULL;
 
   tor_free(the_nodelist);
 }
@@ -959,6 +1039,7 @@ nodelist_ensure_freshness(const networkstatus_t *ns)
     nodelist_set_consensus(ns);
   }
 }
+
 /** Return a list of a node_t * for every node we know about.  The caller
  * MUST NOT modify the list. (You can set and clear flags in the nodes if
  * you must, but you must not add or remove nodes.) */
@@ -2389,7 +2470,6 @@ void
 router_dir_info_changed(void)
 {
   need_to_update_have_min_dir_info = 1;
-  rend_hsdir_routers_changed();
   hs_service_dir_info_changed();
   hs_client_dir_info_changed();
 }
@@ -2740,6 +2820,7 @@ update_router_have_minimum_dir_info(void)
   const networkstatus_t *consensus =
     networkstatus_get_reasonably_live_consensus(now,usable_consensus_flavor());
   int using_md;
+  static int be_loud_when_things_work_again = 0;
 
   if (!consensus) {
     if (!networkstatus_get_latest_consensus())
@@ -2795,8 +2876,9 @@ update_router_have_minimum_dir_info(void)
   if (res && !have_min_dir_info) {
     control_event_client_status(LOG_NOTICE, "ENOUGH_DIR_INFO");
     control_event_boot_dir(BOOTSTRAP_STATUS_ENOUGH_DIRINFO, 0);
-    log_info(LD_DIR,
-             "We now have enough directory information to build circuits.");
+    tor_log(be_loud_when_things_work_again ? LOG_NOTICE : LOG_INFO, LD_DIR,
+            "We now have enough directory information to build circuits.");
+    be_loud_when_things_work_again = 0;
   }
 
   /* If paths have just become unavailable in this update. */
@@ -2805,6 +2887,10 @@ update_router_have_minimum_dir_info(void)
     tor_log(quiet ? LOG_INFO : LOG_NOTICE, LD_DIR,
         "Our directory information is no longer up-to-date "
         "enough to build circuits: %s", dir_info_status);
+    if (!quiet) {
+      /* remember to do a notice-level log when things come back */
+      be_loud_when_things_work_again = 1;
+    }
 
     /* a) make us log when we next complete a circuit, so we know when Tor
      * is back up and usable, and b) disable some activities that Tor
